/**
 * Title:        Viterbi Decoder Tester
 * Description:  A program to test the Viterbi decoder
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.0
 */

import java.io.*;
import ecc.*;

public class ViterbiTest {
    final static int trunc_len = 32;
    public static void main(String[] args) {
        int N = 500000;
        int x[], i, len;

        len = N + trunc_len;
        x = new int[len];
        x[0] = 1; x[1] = x[2] = x[3] = x[4] = x[5] = 0;

        for (i = 6; i < len; i++)
            x[i] = x[i-5] ^ x[i-6];

        Voyager code = new Voyager();
        int y[][] = code.encode(x);

        Channel ch = new Channel(111);
        double sr[][];
        int dr[][];
        int sx[], dx[];

        for (int stni = 12; stni > 1; stni--) {
            double stn = stni * 0.5;
            double sigma = Math.exp(- stn * Math.log(10) / 20);
            ch.setSeed(11011);
            sr = ch.AWGN(y, sigma);
            sx = code.decode(sr, trunc_len);

            int ssum = 0;
            for (i = 0; i < sx.length; i++)
                if (x[i] != sx[i]) ssum++;
            sx = null;

            dr = ch.AWGN2BSC(sr); sr = null;
            dx = code.decode(dr, trunc_len);
            int dsum = 0;
            for (i = 0; i < dx.length; i++)
                if (x[i] != dx[i]) dsum++;
            dx = null; dr = null;

            System.out.print(ssum);
            System.out.print(", ");
            System.out.print(dsum);
            System.out.print(", ");
            System.out.print(stn);
            System.out.print(", ");
            System.out.println(sigma);
        }
    }
}