/**
 * Title:        Viterbi Decoder Applet
 * Description:  An applet to test the Viterbi decoder
 * Copyright:    Copyright (c) 2001, Ling Li
 * Company:      California Institute of Technology
 * @author Ling Li
 * @email  ling@cs.caltech.edu
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import ecc.*;

public class ViterbiTestApplet extends Applet {
    Channel ch;
    Voyager code;
    final static int trunc_len = 32;

    GridBagLayout gridBagLayout = new GridBagLayout();
    TextField txtN = new TextField();
    Label lblN = new Label();
    Label lblSTN = new Label();
    TextField txtSTN = new TextField();
    Button btnTest = new Button();
    Label lblAWGN_BER = new Label();
    Label lblSigma = new Label();
    TextField txtSigma = new TextField();
    Label lblBSC_BER = new Label();
    TextField txtAWGN_BER = new TextField();
    TextField txtBSC_BER = new TextField();
    Label lblSeed = new Label();
    TextField txtSeed = new TextField();

    /**Construct the applet*/
    public ViterbiTestApplet() {
        ch = new Channel();
        code = new Voyager();
    }

    /**Initialize the applet*/
    public void init() {
        try {
            jbInit();
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    public GridBagConstraints newGridBagConstraints
                           (int gridx, int gridy,
                            int gridwidth, int gridheight,
                            double weightx, double weighty,
                            int anchor, int fill,
                            Insets insets, int ipadx, int ipady) {
        GridBagConstraints grid = new GridBagConstraints();
        grid.gridx = gridx;
        grid.gridy = gridy;
        grid.gridwidth = gridwidth;
        grid.gridheight = gridheight;
        grid.fill = fill;
        grid.ipadx = ipadx;
        grid.ipady = ipady;
        grid.insets = insets;
        grid.anchor  = anchor;
        grid.weightx = weightx;
        grid.weighty = weighty;
        return grid;
    }

    /**Component initialization*/
    private void jbInit() throws Exception {
        lblN.setText("N:");
        lblSTN.setText("Eb/N0 (dB):");
        lblSigma.setText("Sigma:");
        lblSeed.setText("Seed:");
        lblAWGN_BER.setText("AWGN:");
        lblBSC_BER.setText("BSC:");

        txtN.setColumns(8);
        txtSTN.setColumns(10);
        txtSTN.addKeyListener(new java.awt.event.KeyAdapter() {

            public void keyReleased(KeyEvent e) {
                txtSTN_keyReleased(e);
            }
        });
        txtSigma.setColumns(10);
        txtSigma.addKeyListener(new java.awt.event.KeyAdapter() {

            public void keyReleased(KeyEvent e) {
                txtSigma_keyReleased(e);
            }
        });
        txtSeed.setColumns(6);
        txtAWGN_BER.setEditable(false);
        txtAWGN_BER.setColumns(8);
        txtBSC_BER.setEditable(false);
        txtBSC_BER.setColumns(8);

        btnTest.setLabel("Test");
        btnTest.addMouseListener(new java.awt.event.MouseAdapter() {

            public void mouseClicked(MouseEvent e) {
                btnTest_mouseClicked(e);
            }
        });

        this.setLayout(gridBagLayout);

        this.add(lblN, newGridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(txtN, newGridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(lblSeed, newGridBagConstraints(2, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(txtSeed, newGridBagConstraints(3, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(lblSTN, newGridBagConstraints(0, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(txtSTN, newGridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(lblSigma, newGridBagConstraints(2, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 0), 0, 0));
        this.add(txtSigma, newGridBagConstraints(3, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));

        this.add(btnTest, newGridBagConstraints(0, 2, 4, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        this.add(lblAWGN_BER, newGridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(txtAWGN_BER, newGridBagConstraints(1, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(lblBSC_BER, newGridBagConstraints(2, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        this.add(txtBSC_BER, newGridBagConstraints(3, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    }
    /**Get Applet information*/
    public String getAppletInfo() {
        return "Viterbi Decoding Test Applet";
    }
    /**Get parameter info*/
    public String[][] getParameterInfo() {
        return null;
    }

    void updateSigma (double sigma) {
        if (sigma > 0) {
            double stn = - 20 * Math.log(sigma) / Math.log(10);
            txtSTN.setText(String.valueOf(stn));
        }
    }

    void updateSTN (double stn) {
        double sigma = Math.exp(- stn * Math.log(10) / 20);
        txtSigma.setText(String.valueOf(sigma)); //Double.toString(sigma)
    }

    int[] genCode(int N) {
        if (N < 1) return null;

        int x[], i, len;

        len = N + trunc_len;
        x = new int[len];
        x[0] = 1; x[1] = x[2] = x[3] = x[4] = x[5] = 0;

        for (i = 6; i < len; i++)
            x[i] = x[i-5] ^ x[i-6];

        return x;
    }

    void btnTest_mouseClicked(MouseEvent e) {
        int N, seed;
        double stn, sigma;

        txtAWGN_BER.setText("n/a");
        txtBSC_BER.setText("n/a");

        try {
            N = Integer.parseInt(txtN.getText());
            seed = Integer.parseInt(txtSeed.getText());
            sigma = Double.valueOf(txtSigma.getText()).doubleValue();
        } catch (NumberFormatException ex) {
            return;
        }


        if (N < 1)
            return;

        int x[] = genCode(N);
        int y[][] = code.encode(x);

        ch.setSeed(seed);

        double sr[][] = ch.AWGN(y, sigma);
        int dr[][] = ch.AWGN2BSC(sr);

        int sx[] = code.decode(sr, trunc_len);
        int dx[] = code.decode(dr, trunc_len);

        int ssum = 0, dsum = 0;
        for (int i = 0; i < sx.length; i++) {
            if (x[i] != sx[i]) ssum++;
            if (x[i] != dx[i]) dsum++;
        }

        txtAWGN_BER.setText(String.valueOf(ssum / (double)N));
         txtBSC_BER.setText(String.valueOf(dsum / (double)N));
    }

    void txtSTN_keyReleased(KeyEvent e) {
        updateSTN(Double.valueOf(txtSTN.getText()).doubleValue());
    }

    void txtSigma_keyReleased(KeyEvent e) {
        updateSigma(Double.valueOf(txtSigma.getText()).doubleValue());
    }
}
